/********************************************************************
*                   LIBRERIA USO BUS ONEWIRE
* 
*   - Funciones para comunicacion onewire
*   - Hay dos velocidades, pero esta libreria usa la mas lenta 15.4Kbps 
*   - Hay mucha informacin sobre este bus:
*       - https://www.maximintegrated.com/en/app-notes/index.mvp/id/126
*       - https://datasheets.maximintegrated.com/en/ds/DS18S20.pdf
*       - http://ww1.microchip.com/downloads/en/AppNotes/01199a.pdf  
 *******************************************************************/

/* 
 * File:   
 * Author: Eduardo
 * Comments:
 * Revision history: 22/12/17
 */

#include "OneWire.h"
#include <xc.h> 

/******************************************************************************
*                       DEFINIMOS EL PIN A USAR
* 
*   - Asigna un pin al bus
 *****************************************************************************/

#define OW_DataPin TRISDbits.TRISD7     // Configura como entrada o salida
#define OW_DataInput  PORTDbits.RD7     // Para leer estado
#define OW_DataOutput LATDbits.LATD7    // Para escribir estado



/******************************************************************************
*                       FUNCION INICIALIZAR BUS
* 
*   - 
 *****************************************************************************/

void OW_begin ()
{

}



/******************************************************************************
*                       FUNCION ENVIO RESET BUS
* 
*   - Realiza un reset para llamar la atencin de los esclavos en el bus
*   - Devuelve true, si algn esclavo responde
 *****************************************************************************/

bool reset_OW ()
{
    OW_DataPin = 1;             // Habilita el pin como entrada
    while (!OW_DataInput);      // Espera hasta que el bus este en reposo
    bool detectado = false;     // Inicializamos una variable que nos indica si 
                                // algun esclavo responde
    
    drive_OW_low ();            // Pone el bus a nivel bajo durante 480uS
    OW_delayUs(480); 
    drive_OW_high ();           // Pone el bus en reposo durante 70uS
    OW_delayUs(70); 
    
    OW_DataPin = 1;             // Configura el pin como entada otra vez. Las 
                                // funciones anteriores lo habian cambiado a salida
    
    if (OW_DataInput)           // Monitoriza el bus para detectar una respuesta
        detectado = false;
    else
        detectado = true;
    
    OW_delayUs(410);              // Realiza una espra de 410uS esperando a finalizar
                                // el proceso de reset
    return detectado;           // Devuelve si ha detectado o no un esclavo
}



/******************************************************************************
*                       FUNCION PONE BUS A NIVEL ALTO
* 
*   - Configura el pin como salida y pone el bus a nivel alto que es el estado
*   en reposo 
 *****************************************************************************/

void drive_OW_high ()
{
  OW_DataPin = 0;               // Habilita el pin como salida
  OW_DataOutput = 1;            // Pone el bus a nivel alto
}



/******************************************************************************
*                       FUNCION PONE BUS A NIVEL BAJO
* 
*   - Configura el pin como salida y pone el bus a nivel bajo
 *****************************************************************************/

void drive_OW_low ()
{
  OW_DataPin = 0;               // Habilita el pin como salida
  OW_DataOutput = 0;            // Pone el bus a nivel bajo
}



/******************************************************************************
*                       FUNCION ENVIA BIT 0 o 1
* 
*   - Realiza la secuencia para transmitir un bit 0 o 1
*   - Si recibe true, envia 1, si recibe false, envia 0 
 *****************************************************************************/

void write_OW_bit (bool valor)
{
  if (valor)                    // Si el valor es true, enviamos un uno
  {
       drive_OW_low ();         // Pone el bus a nivel baja durante 6uS
       OW_delayUs(6);
       drive_OW_high ();        // Pone el bus en reposo durante 64uS
       OW_delayUs(64);
  }
  else                          // Si el valor es false, enviamos un cero
  {
       drive_OW_low ();         // Pone el bus a nivel bajo durante 60uS
       OW_delayUs(60);
       drive_OW_high ();        // Pone el bus en reposo durante 10uS
       OW_delayUs(10);
  }
}




/******************************************************************************
*                       FUNCION LEE DEL BUS
* 
*   - Realiza la secuencia para leer del bus
*   - Devuelve true o false segn haya leido 1 o 0
 *****************************************************************************/

bool read_OW_bit ()
{
       bool detectado = false;  // Inicializamos una variable que nos indica si 
                                // ha leido 1 o 0
    
       drive_OW_low ();         // Enviamos un pulso a modo de reloj al esclavo
       OW_delayUs(6);             // poniendo el bus a nivel bajo durante 6 uS
       drive_OW_high ();        // Luego, el bus pasa a reposo
       
       OW_DataPin = 1;          // Configuramos el pin como entrada para leer
       
       OW_delayUs(5);             // Esperamos 5uS y leemos. En el datasheet pone
                                // 9uS, pero a veces ya habia pasado el evento
                                // supongo que por el retardo de poner el pin
                                // como entrada
       
       detectado = OW_DataInput;// Cargamos en la variable el valor leido
       
       OW_delayUs(59);            // Realizamos un retardo de 59uS para completar
                                // el proceso. En el datasheet pone 55uS, aadimos
                                // los 4uS que restamos antes.

       return detectado;        // Devuelve el valor detectado.
           
}



/******************************************************************************
*                       FUNCION ESCRIBE BYTE EN EL BUS
* 
*   - Realiza la secuencia para transmitir 8 bits
*   - Recibe como parametro el byte a transmitir
*   - Trasmite primero el bit de menor peso 
 *****************************************************************************/

void write_OW_byte (uint8_t valor)
{
    for (uint8_t temp = 0; temp < 8; temp ++)
    {                               // Recorremos bit a bit el byte e transmitimos
       if (valor & 0x01 )           // cada valor
           write_OW_bit(true);
       else
           write_OW_bit(false);
       valor >>= 1;
    }   
}



/******************************************************************************
*                       FUNCION LEE UN BYTE DEL BUS
* 
*   - Realiza la secuencia para leer 8 bits del bus
*   - Devuelve el byte leido
 *****************************************************************************/

uint8_t read_OW_byte ()
{
    uint8_t byteLeido = 0;    // Variable que almacenara el byte recibido
    
    for (uint8_t temp = 0; temp < 8; temp ++)
    {
       byteLeido >>=1;              // Vamos recogiendo cada byte y los vamos
       if (read_OW_bit ())          // desplazando a la izquierda para tener
       {                            // el de menor peso en su posicion
           byteLeido = byteLeido | 0x80;
       }
       else 
       {
           byteLeido = byteLeido & 0x7F;
       }
    }
    
    return byteLeido;
}



/******************************************************************************
*                       FUNCION RETARDO US
* 
*   - Realiza un retardo de uS segun el valor recibido
 *****************************************************************************/
void OW_delayUs(uint16_t i)
{
    for ( uint16_t x=0; x<i; x++ ) 
    { 
    __delay_us(1); 
    }
}